from unittest import TestCase

from src.jwksutils import ensure_bytes


class TestEnsureBytes(TestCase):

    def test_should_return_bytes_when_is_str(self):
        self.assertEqual(ensure_bytes('asdf'), b'asdf')

    def test_should_return_bytes_when_is_not_str(self):
        self.assertEqual(ensure_bytes(b'asdf'), b'asdf')



from eve.auth import TokenAuth
from flask import current_app as app

from .verifier import AzureVerifier


class AzureAuth(TokenAuth):

    def get_configs(self):
        tenant = app.config.get('AZURE_AD_TENANT')
        issuer = app.config['AZURE_AD_ISSUER']
        audiences = app.config['AZURE_AD_AUDIENCES']

        return dict(tenant=tenant, issuer=issuer, audiences=audiences)

    def validate_token(self, token):
        return AzureVerifier(**self.get_configs()).verify(token)

    def check_auth(self, token, allowed_roles, resource, method):
        return self.validate_token(token)



__title__ = 'Eve-Azure-Auth'
__description__ = ''
__url__ = 'https://github.com/ateliedocodigo/eve-azure-auth'
__version__ = '0.2.0'
__author__ = 'LuisComS'
__author_email__ = 'luiscoms@ateliedocodigo.com.br'
__copyright__ = 'Copyright 2020 Ateliê do Código'



from unittest import TestCase
from unittest.mock import MagicMock, patch

from src import AzureAuth
from src.verifier import AzureVerifier


class TestAzureAuthCheckAuth(TestCase):

    @patch.object(AzureAuth, 'validate_token')
    def test_should_call_validate_token(self, validate_token_mock):
        AzureAuth().check_auth('asdf', None, None, None)
        validate_token_mock.assert_called_once_with('asdf')


class TestAzureAuthValidateToken(TestCase):

    @patch.object(AzureAuth, 'get_configs', MagicMock())
    @patch.object(AzureVerifier, 'verify')
    def test_should_call_auth_verifier(self, verify_mock):
        AzureAuth().validate_token('asdf')
        verify_mock.assert_called_once_with('asdf')



import base64
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicNumbers
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization


def ensure_bytes(key):
    if isinstance(key, str):
        key = key.encode('utf-8')
    return key


def decode_value(val):
    decoded = base64.urlsafe_b64decode(ensure_bytes(val) + b'==')
    return int.from_bytes(decoded, 'big')


def rsa_pem_from_jwk(jwk):
    return RSAPublicNumbers(
        n=decode_value(jwk['n']),
        e=decode_value(jwk['e'])
    ).public_key(default_backend()).public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo
    )



from setuptools import setup


def get_info(info):
    for line in open('src/__version__.py').readlines():
        if line.startswith(info):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]

    raise RuntimeError('Unable to find version string.')


with open('README.md', 'r') as f:
    readme = f.read()

reqs = [
    'cachecontrol<1',
    'cryptography<3',
    'PyJWT<2',
    'eve<2',
]

setup(
    name=get_info('__title__'),
    version=get_info('__version__'),
    description=get_info('__description__'),
    long_description=readme,
    long_description_content_type='text/markdown',
    author=get_info('__author__'),
    author_email=get_info('__author_email__'),
    url=get_info('__url__'),
    install_requires=reqs,
    zip_safe=False,
    include_package_data=True,
    exclude_package_data={'': ['README.md']},
    package_dir={'eve_azure_ad_auth': 'src'},
    packages=['eve_azure_ad_auth'],
    python_requires='>= 3.6',
)



import jwt
import requests
from cachecontrol import CacheControl

from .jwksutils import rsa_pem_from_jwk


class InvalidAuthorizationToken(Exception):
    def __init__(self, details):
        super().__init__('Invalid authorization token: ' + details)


class AzureVerifier:
    api_version = 'v2.0'
    authority = 'https://login.microsoftonline.com'
    tenant = 'common'
    discovery_document = '.well-known/openid-configuration'

    audiences = None
    issuer = None

    def __init__(self, tenant='common', issuer=None, audiences=None):
        self.tenant = tenant or self.tenant
        self.issuer = issuer
        self.audiences = audiences
        self.session = CacheControl(requests.Session())

    @property
    def documents_uri(self):
        return f'{self.authority}/{self.tenant}/{self.api_version}/{self.discovery_document}'

    @property
    def jwks_uri(self):
        return self.session.get(self.documents_uri).json()['jwks_uri']

    @property
    def jwks(self):
        return self.session.get(self.jwks_uri).json()

    def get_jwk(self, kid):
        for jwk in self.jwks.get('keys'):
            if jwk.get('kid') == kid:
                return jwk
        raise InvalidAuthorizationToken('kid not recognized')

    def get_kid(self, token):
        headers = jwt.get_unverified_header(token)
        if not headers:
            raise InvalidAuthorizationToken('missing headers')
        try:
            return headers['kid']
        except KeyError:
            raise InvalidAuthorizationToken('missing kid')

    def get_public_key(self, token):
        return rsa_pem_from_jwk(self.get_jwk(self.get_kid(token)))

    def verify(self, token):
        public_key = self.get_public_key(token)

        return jwt.decode(
            token, public_key, verify=True, algorithms=['RS256'], audience=self.audiences, issuer=self.issuer
        )
